@@ -19,13 +19,23 @@
 #
 # Author: Don Welch
 
+# Local
 from base.g import *
 from base import utils
 
+# StdLib
 import os.path
 import re
 import glob
 
+try:
+    import datetime
+    datetime_avail = True
+except ImportError:
+    datetime_avail = False
+    datetime = None
+
+
 pat_prod_num = re.compile("""(\d+)""", re.I)
 
 TYPE_UNKNOWN = 0
@@ -36,6 +46,9 @@ TYPE_BOOL = 3
 TYPE_INT = 4
 TYPE_HEX = 5
 TYPE_BITFIELD = 6
+TYPE_URI = TYPE_STR # (7) not used (yet)
+TYPE_DATE = 8  # format: mm/dd/yyyy
+
 
 TECH_CLASSES = [
     "Undefined", # This will show an error (and its the default)
@@ -75,6 +88,7 @@ TECH_CLASSES = [
     "QuickConnect",
     "DJ55xx",
     "OJProKx50",
+    'LJP1XXX',
 ]
 
 TECH_CLASSES.sort()
@@ -88,7 +102,7 @@ TECH_CLASS_PDLS = {
     "LJColor"      : 'pcl3',
     "LJFastRaster" : 'pclxl',
     "LJJetReady"   : 'pclxl',
-    "DJ350"        : 'pcl3', 
+    "DJ350"        : 'pcl3',
     #"DJ400"        : 'pcl3',
     "DJ540"        : 'pcl3',
     "DJ600"        : 'pcl3',
@@ -112,10 +126,34 @@ TECH_CLASS_PDLS = {
     "PSP470"       : 'pcl3',
     "LJZjsMono"    : 'zjs',
     "LJZjsColor"   : 'zjs',
-    "LJm1005"      : 'zxstream',
+    "LJm1005"      : 'zxs',
     "QuickConnect" : 'jpeg',
     "DJ55xx"       : 'pcl3',
     "OJProKx50"    : 'pcl3',
+    'LJP1XXX'      : 'zxs',
+}
+
+PDL_TYPE_PCL = 0  # less preferred
+PDL_TYPE_PS = 1   #      /\
+PDL_TYPE_HOST = 2 # more preferred (however, may req. plugin)
+
+PDL_TYPES = { # Used to prioritize PPD file selection in prnt.cups.getPPDFile2()
+    'pcl3' : PDL_TYPE_PCL,
+    'pcl5' : PDL_TYPE_PCL,
+    'pcl6' : PDL_TYPE_PCL,
+    'pcl5e' : PDL_TYPE_PCL,
+    'pcl' : PDL_TYPE_PCL,
+    'pclxl' : PDL_TYPE_PCL,
+    'ps' : PDL_TYPE_PS,
+    'lidil' : PDL_TYPE_HOST,
+    'zjs' : PDL_TYPE_HOST,
+    'zjstream' : PDL_TYPE_HOST,
+    'zxs' : PDL_TYPE_HOST,
+    'zxstream' : PDL_TYPE_HOST,
+    'jpeg' : PDL_TYPE_HOST,
+    'jpg' : PDL_TYPE_HOST,
+    'jetready' : PDL_TYPE_HOST,
+    'jr' : PDL_TYPE_HOST,
 }
 
 
@@ -144,25 +182,14 @@ TECH_SUBCLASSES.sort()
 # Items will be capitalized unless in this dict
 MODEL_UI_REPLACEMENTS = {'laserjet'   : 'LaserJet',
                           'psc'        : 'PSC',
-                          'officejet'  : 'Officejet',
-                          'deskjet'    : 'Deskjet',
                           'hp'         : 'HP',
-                          'business'   : 'Business',
-                          'inkjet'     : 'Inkjet',
-                          'photosmart' : 'Photosmart',
-                          'color'      : 'Color',
-                          'series'     : 'series',
-                          'printer'    : 'Printer',
                           'mfp'        : 'MFP',
-                          'mopier'     : 'Mopier',
-                          'pro'        : 'Pro',
-                          'apollo'     : 'Apollo',
                         }
-        
-        
+
+
 def normalizeModelUIName(model):
     ml = model.lower().strip()
-    
+
     if 'apollo' in ml:
         z = ml.replace('_', ' ')
     else:
@@ -170,7 +197,7 @@ def normalizeModelUIName(model):
             z = ml[3:].replace('_', ' ')
         else:
             z = ml.replace('_', ' ')
-    
+
     y = []
     for x in z.split():
         if pat_prod_num.search(x): # don't cap items like cp1700dn
@@ -182,11 +209,11 @@ def normalizeModelUIName(model):
         return ' '.join(y)
     else:
         return "HP " + ' '.join(y)
-        
+
 
 def normalizeModelName(model):
     return utils.xstrip(model.replace(' ', '_').replace('__', '_').replace('~','').replace('/', '_'), '_')
-        
+
 
 class ModelData:
     def __init__(self, root_path=None):
@@ -198,11 +225,26 @@ class ModelData:
         self.__cache = {}
         self.reset_includes()
         self.sec = re.compile(r'^\[(.*)\]')
-        self.inc = re.compile(r'^\%include (.*)', re.IGNORECASE)
+        self.inc = re.compile(r'^\%include (.*)', re.I)
         self.inc_line = re.compile(r'^\%(.*)\%')
         self.eq = re.compile(r'^([^=]+)=(.*)')
-        
+        self.date = re.compile(r'^(\d{1,2})/(\d{1,2})/(\d{4,4})')
+
+        files = [(os.path.join(self.root_path, "models.dat"),
+                  os.path.join(self.root_path, "unreleased", "unreleased.dat")),
+                 (os.path.join(os.getcwd(), 'data', 'models', 'models.dat'),
+                  os.path.join(os.getcwd(), 'data', 'models', 'unreleased', 'unreleased.dat'))]
+
+        for self.released_dat, self.unreleased_dat in files:
+            if os.path.exists(self.released_dat):
+                break
+
+        else:
+            self.released_dat, self.unreleased_dat = None, None
+            log.error("Unable to locate models.dat file")
+
         self.FIELD_TYPES = {
+            # Static model query data (from models.dat)
             'align-type' : TYPE_INT,
             'clean-type' : TYPE_INT,
             'color-cal-type' : TYPE_INT,
@@ -214,10 +256,12 @@ class ModelData:
             'io-mfp-mode' : TYPE_INT,
             'io-mode' : TYPE_INT,
             'io-support' : TYPE_BITFIELD,
+            'monitor-type' : TYPE_INT,
             'linefeed-cal-type' : TYPE_INT,
             'panel-check-type' : TYPE_INT,
             'pcard-type' : TYPE_INT,
             'plugin' : TYPE_INT,
+            'plugin-reason' : TYPE_BITFIELD,
             'power-settings': TYPE_INT,
             'pq-diag-type' : TYPE_INT,
             'r-type' : TYPE_INT,
@@ -226,6 +270,7 @@ class ModelData:
             'status-battery-check' : TYPE_INT,
             'status-dynamic-counters' : TYPE_INT,
             'status-type' : TYPE_INT,
+            'support-subtype' : TYPE_HEX,
             'support-released' : TYPE_BOOL,
             'support-type' : TYPE_INT,
             'support-ver' : TYPE_STR,
@@ -235,32 +280,74 @@ class ModelData:
             'usb-pid' : TYPE_HEX,
             'usb-vid' : TYPE_HEX,
             'job-storage' : TYPE_INT,
+
+            # Dynamic model data (from device query)
+            'dev-file' : TYPE_STR,
+            'fax-uri' : TYPE_STR,
+            'scan-uri' : TYPE_STR,
+            'is-hp' : TYPE_BOOL,
+            'host' : TYPE_STR,
+            'status-desc' : TYPE_STR,
+            'cups-printer' : TYPE_STR,
+            'serial' : TYPE_STR,
+            'error-state' : TYPE_INT,
+            'device-state' : TYPE_INT,
+            'panel' : TYPE_INT,
+            'device-uri' : TYPE_STR,
+            'panel-line1' : TYPE_STR,
+            'panel-line2' : TYPE_STR,
+            'back-end' : TYPE_STR,
+            'port' : TYPE_INT,
+            'deviceid' : TYPE_STR,
+            'cups-uri' : TYPE_STR,
+            'status-code' : TYPE_INT,
+            'rs' : TYPE_STR,
+            'rr' : TYPE_STR,
+            'rg' : TYPE_STR,
+            'r' : TYPE_INT,
+            'duplexer' : TYPE_INT,
+            'supply-door' : TYPE_INT,
+            'revision' : TYPE_INT,
+            'media-path' : TYPE_INT,
+            'top-door' : TYPE_BOOL,
+            'photo-tray' : TYPE_BOOL,
             }
-            
+
         self.RE_FIELD_TYPES = {
-            re.compile('r(\d+)-agent(\d+)-kind', re.IGNORECASE) : TYPE_INT,
-            re.compile('r(\d+)-agent(\d+)-type', re.IGNORECASE) : TYPE_INT,
-            re.compile('r(\d+)-agent(\d+)-sku', re.IGNORECASE) : TYPE_STR,
+            re.compile('^r(\d+)-agent(\d+)-kind', re.IGNORECASE) : TYPE_INT,
+            re.compile('^r(\d+)-agent(\d+)-type', re.IGNORECASE) : TYPE_INT,
+            re.compile('^r(\d+)-agent(\d+)-sku', re.IGNORECASE) : TYPE_STR,
+            re.compile('^agent(\d+)-desc', re.IGNORECASE) : TYPE_STR,
+            re.compile('^agent(\d+)-virgin', re.IGNORECASE) : TYPE_BOOL,
+            re.compile('^agent(\d+)-dvc', re.IGNORECASE) : TYPE_INT,
+            re.compile('^agent(\d+)-kind', re.IGNORECASE) : TYPE_INT,
+            re.compile('^agent(\d+)-type', re.IGNORECASE) : TYPE_INT,
+            re.compile('^agent(\d+)-id', re.IGNORECASE) : TYPE_INT,
+            re.compile('^agent(\d+)-hp-ink', re.IGNORECASE) : TYPE_BOOL,
+            re.compile('^agent(\d+)-health-desc', re.IGNORECASE) : TYPE_STR,
+            re.compile('^agent(\d+)-health$', re.IGNORECASE) : TYPE_INT,
+            re.compile('^agent(\d+)-known', re.IGNORECASE) : TYPE_BOOL,
+            re.compile('^agent(\d+)-level', re.IGNORECASE) : TYPE_INT,
+            re.compile('^agent(\d+)-ack', re.IGNORECASE) : TYPE_BOOL,
+            re.compile('^agent(\d+)-sku', re.IGNORECASE) : TYPE_STR,
+            re.compile('^in-tray(\d+)', re.IGNORECASE) : TYPE_BOOL,
+            re.compile('^out-tray(\d+)', re.IGNORECASE) : TYPE_BOOL,
             re.compile('model(\d+)', re.IGNORECASE) : TYPE_STR,
             }
-            
+
         self.TYPE_CACHE = {}
-            
-          
+
 
     def read_all_files(self, unreleased=True):
-        released_dat = os.path.join(self.root_path, "models.dat")
-        log.debug("Reading file: %s" % released_dat)
-        self.read_section(released_dat)
+        if os.path.exists(self.released_dat):
+            self.read_section(self.released_dat)
 
-        unreleased_dir = os.path.join(self.root_path, 'unreleased')
-        if unreleased and os.path.exists(unreleased_dir):
-            unreleased_dat = os.path.join(self.root_path, "unreleased", "unreleased.dat")
-            log.debug("Reading file: %s" % unreleased_dat)
-            self.read_section(unreleased_dat)
+            if self.unreleased_dat is not None and os.path.exists(self.unreleased_dat):
+                self.read_section(self.unreleased_dat )
 
         return self.__cache
 
+
     def read_section(self, filename, section=None, is_include=False): # section==None, read all sections
         found, in_section = False, False
 
@@ -326,7 +413,7 @@ class ModelData:
                 if in_section:
                     match = self.inc_line.match(line)
 
-                    if match is not None: 
+                    if match is not None:
                         inc_sect = match.group(1)
                         log.debug("Found include directive %%%s%%" % inc_sect)
 
@@ -340,30 +427,19 @@ class ModelData:
                             else:
                                 log.error("Include %%%s%% not found." % inc_sect)
 
-
             if in_section:
                 match = self.eq.search(line)
 
                 if match is not None:
                     key = match.group(1)
                     value = match.group(2)
-                    typ = self.get_data_type(key)
-                    
-                    if  typ in (TYPE_BITFIELD, TYPE_INT):
-                        value = int(value)
-                    
-                    elif typ == TYPE_BOOL:
-                        #value = utils.to_bool(value)
-                        value = int(value)
-                        
-                    elif typ == TYPE_LIST:
-                        value = [x for x in value.split(',') if x]
-                        
+                    value = self.convert_data(key, value)
                     cache[read_section][key] = value
 
         fd.close()
         return found
 
+
     def reset_includes(self):
         self.__include_files = []
         self.__includes = {}
@@ -377,19 +453,15 @@ class ModelData:
         except:
             log.debug("Cache miss: %s" % model)
 
-            released_dat = os.path.join(self.root_path, "models.dat")
-            log.debug("Reading file: %s" % released_dat)
+            log.debug("Reading file: %s" % self.released_dat)
 
-            if self.read_section(released_dat, model):
+            if self.read_section(self.released_dat, model):
                 return self.__cache[model]
 
-            unreleased_dir = os.path.join(self.root_path, 'unreleased')
+            if self.unreleased_dat is not None and os.path.exists(self.unreleased_dat):
+                log.debug("Reading file: %s" % self.unreleased_dat)
 
-            if os.path.exists(unreleased_dir):
-                unreleased_dat = os.path.join(self.root_path, "unreleased", "unreleased.dat")
-                log.debug("Reading file: %s" % unreleased_dat)
-
-                if self.read_section(unreleased_dat, model):
+                if self.read_section(self.unreleased_dat, model):
                     return self.__cache[model]
 
             return {}
@@ -397,7 +469,8 @@ class ModelData:
 
     def all_models(self):
         return self.__cache
-        
+
+
     def get_data_type(self, key):
         try:
             return self.FIELD_TYPES[key]
@@ -410,7 +483,45 @@ class ModelData:
                     if match is not None:
                         self.TYPE_CACHE[key] = typ
                         return typ
-        
-        return TYPE_STR
-    
 
+        log.warn("get_data_type(): Defaulted to TYPE_STR for key %s" % key)
+        return TYPE_STR
+
+
+    def convert_data(self, key, value, typ=None):
+        if typ is None:
+            typ = self.get_data_type(key)
+
+        if  typ in (TYPE_BITFIELD, TYPE_INT):
+            try:
+                value = int(value)
+            except (ValueError, TypeError):
+                log.error("Invalid value in .dat file: %s=%s" % (key, value))
+                value = 0
+
+        elif typ == TYPE_BOOL:
+            value = utils.to_bool(value)
+
+        elif typ == TYPE_LIST:
+            value = [x for x in value.split(',') if x]
+
+        elif typ == TYPE_DATE: # mm/dd/yyyy
+            if datetime_avail:
+                # ...don't use datetime.strptime(), wasn't avail. until 2.5
+                match = self.date.search(value)
+
+                if match is not None:
+                    month = int(match.group(1))
+                    day = int(match.group(2))
+                    year = int(match.group(3))
+
+                    value = datetime.date(year, month, day)
+
+        elif typ == TYPE_HEX:
+            try:
+                value = int(value, 16)
+            except (ValueError, TypeError):
+                log.error("Invalid hex value in .dat file: %s=%s" % (key, value))
+                value = 0
+
+        return value
