@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# (c) Copyright 2003-2007 Hewlett-Packard Development Company, L.P.
+# (c) Copyright 2003-2009 Hewlett-Packard Development Company, L.P.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,9 +20,10 @@
 # Author: Don Welch
 #
 
-__version__ = '14.0'
+__version__ = '14.1'
 __title__ = 'Dependency/Version Check Utility'
-__doc__ = "Check the existence and versions of HPLIP dependencies."
+__mod__ = 'hp-check'
+__doc__ = """Check the existence and versions of HPLIP dependencies. (Run as 'python ./check.py' from the HPLIP tarball before installation.)"""
 
 # Std Lib
 import sys
@@ -40,6 +41,7 @@ from installer.core_install import *
 device_avail = False
 try:
     from base import device, pml
+    # This can fail due to hpmudext not being present
 except ImportError:
     log.debug("Device library is not avail.")
 else:
@@ -47,7 +49,7 @@ else:
 
 
 USAGE = [(__doc__, "", "name", True),
-         ("Usage: hp-check/check.py [OPTIONS]", "", "summary", True),
+         ("Usage: %s [OPTIONS]" % __mod__, "", "summary", True),
          utils.USAGE_OPTIONS,
          ("Compile-time check:", "-c or --compile", "option", False),
          ("Run-time check:", "-r or --run", "option", False),
@@ -56,9 +58,9 @@ USAGE = [(__doc__, "", "name", True),
          utils.USAGE_LOGGING_PLAIN,
          utils.USAGE_HELP,
          utils.USAGE_NOTES,
-         ("1. For checking for the proper build environment for the HPLIP supplied tarball (.tar.gz or .run),", "", "note", False), 
+         ("1. For checking for the proper build environment for the HPLIP supplied tarball (.tar.gz or .run),", "", "note", False),
          ("use the --compile or --both switches.", "", "note", False),
-         ("2. For checking for the proper runtime environment for a distro supplied package (.deb, .rpm, etc),", "", "note", False), 
+         ("2. For checking for the proper runtime environment for a distro supplied package (.deb, .rpm, etc),", "", "note", False),
          ("use the --runtime switch.", "", "note", False),
         ]
 
@@ -66,8 +68,8 @@ def usage(typ='text'):
     if typ == 'text':
         utils.log_title(__title__, __version__)
 
-    utils.format_text(USAGE, typ, __title__, 'hp-check', __version__)
-    sys.exit(0)        
+    utils.format_text(USAGE, typ, __title__, __mod__, __version__)
+    sys.exit(0)
 
 
 build_str = "HPLIP will not build, install, and/or function properly without this dependency."
@@ -113,12 +115,12 @@ overall_commands_to_run = []
 time_flag = DEPENDENCY_RUN_AND_COMPILE_TIME
 
 try:
-    log.set_module("hp-check")
+    log.set_module(__mod__)
 
     try:
-        opts, args = getopt.getopt(sys.argv[1:], 'hl:gtcrb', 
-            ['help', 'help-rest', 'help-man', 'help-desc', 'logging=', 
-             'run', 'runtime', 'compile', 'both']) 
+        opts, args = getopt.getopt(sys.argv[1:], 'hl:gtcrb',
+            ['help', 'help-rest', 'help-man', 'help-desc', 'logging=',
+             'run', 'runtime', 'compile', 'both'])
 
     except getopt.GetoptError, e:
         log.error(e.msg)
@@ -152,13 +154,13 @@ try:
 
         elif o == '-t':
             fmt = False
-            
+
         elif o in ('-c', '--compile'):
             time_flag = DEPENDENCY_COMPILE_TIME
-            
+
         elif o in ('-r', '--runtime', '--run'):
             time_flag = DEPENDENCY_RUN_TIME
-            
+
         elif o in ('-b', '--both'):
             time_flag = DEPENDENCY_RUN_AND_COMPILE_TIME
 
@@ -171,19 +173,18 @@ try:
     utils.log_title(__title__, __version__)
 
     log.info(log.bold("Note: hp-check can be run in three modes:"))
-    
+
     for l in tui.format_paragraph("1. Compile-time check mode (-c or --compile): Use this mode before compiling the HPLIP supplied tarball (.tar.gz or .run) to determine if the proper dependencies are installed to successfully compile HPLIP."):
         log.info(l)
-        
+
     for l in tui.format_paragraph("2. Run-time check mode (-r or --run): Use this mode to determine if a distro supplied package (.deb, .rpm, etc) or an already built HPLIP supplied tarball has the proper dependencies installed to successfully run."):
         log.info(l)
-    
+
     for l in tui.format_paragraph("3. Both compile- and run-time check mode (-b or --both) (Default): This mode will check both of the above cases (both compile- and run-time dependencies)."):
         log.info(l)
-    
+
     log.info()
-    
-    
+
     log_file = os.path.normpath('./hp-check.log')
     log.info(log.bold("Saving output in log file: %s" % log_file))
     log.debug("Log file=%s" % log_file)
@@ -196,6 +197,7 @@ try:
     log.info("\nInitializing. Please wait...")
     core =  CoreInstall(MODE_CHECK)
     core.init()
+    core.set_plugin_version()
 
     tui.header("SYSTEM INFO")
 
@@ -231,69 +233,88 @@ try:
             sys.exit(1)
 
 
-    log.info()
-    log.info(log.bold("Checking PyQt version..."))
+    if core.ui_toolkit == 'qt3':
 
-    # PyQt
-    try:
-        import qt
-    except ImportError:
-        num_errors += 1
-        log.error("NOT FOUND OR FAILED TO LOAD!")
-    else:
-        # check version of Qt
-        qtMajor = int(qt.qVersion().split('.')[0])
+        log.info()
+        log.info(log.bold("Checking PyQt 3.x version..."))
 
-        if qtMajor < MINIMUM_QT_MAJOR_VER:
-            log.error("Incorrect version of Qt installed. Ver. 3.0.0 or greater required.")
+        # PyQt 3
+        try:
+            import qt
+        except ImportError:
+            num_errors += 1
+            log.error("NOT FOUND OR FAILED TO LOAD!")
         else:
-            #check version of PyQt
-            try:
-                pyqtVersion = qt.PYQT_VERSION_STR
-            except AttributeError:
-                pyqtVersion = qt.PYQT_VERSION
+            # check version of Qt
+            qtMajor = int(qt.qVersion().split('.')[0])
 
-            while pyqtVersion.count('.') < 2:
-                pyqtVersion += '.0'
-
-            (maj_ver, min_ver, pat_ver) = pyqtVersion.split('.')
-
-            if pyqtVersion.find('snapshot') >= 0:
-                log.error("A non-stable snapshot version of PyQt is installed (%s)." % pyqtVersion)
-                num_errors += 1
+            if qtMajor < MINIMUM_QT_MAJOR_VER:
+                log.error("Incorrect version of Qt installed. Ver. 3.0.0 or greater required.")
             else:
+                #check version of PyQt
                 try:
-                    maj_ver = int(maj_ver)
-                    min_ver = int(min_ver)
-                    pat_ver = int(pat_ver)
-                except ValueError:
-                    maj_ver, min_ver, pat_ver = 0, 0, 0
+                    pyqtVersion = qt.PYQT_VERSION_STR
+                except AttributeError:
+                    pyqtVersion = qt.PYQT_VERSION
 
-                if maj_ver < MINIMUM_PYQT_MAJOR_VER or \
-                    (maj_ver == MINIMUM_PYQT_MAJOR_VER and min_ver < MINIMUM_PYQT_MINOR_VER):
+                while pyqtVersion.count('.') < 2:
+                    pyqtVersion += '.0'
+
+                (maj_ver, min_ver, pat_ver) = pyqtVersion.split('.')
+
+                if pyqtVersion.find('snapshot') >= 0:
+                    log.error("A non-stable snapshot version of PyQt is installed (%s)." % pyqtVersion)
                     num_errors += 1
-                    log.error("HPLIP may not function properly with the version of PyQt that is installed (%d.%d.%d)." % (maj_ver, min_ver, pat_ver))
-                    log.error("Ver. %d.%d or greater required." % (MINIMUM_PYQT_MAJOR_VER, MINIMUM_PYQT_MINOR_VER))
                 else:
-                    log.info("OK, version %d.%d installed." % (maj_ver, min_ver))
+                    try:
+                        maj_ver = int(maj_ver)
+                        min_ver = int(min_ver)
+                        pat_ver = int(pat_ver)
+                    except ValueError:
+                        maj_ver, min_ver, pat_ver = 0, 0, 0
+
+                    if maj_ver < MINIMUM_PYQT_MAJOR_VER or \
+                        (maj_ver == MINIMUM_PYQT_MAJOR_VER and min_ver < MINIMUM_PYQT_MINOR_VER):
+                        num_errors += 1
+                        log.error("HPLIP may not function properly with the version of PyQt that is installed (%d.%d.%d)." % (maj_ver, min_ver, pat_ver))
+                        log.error("Ver. %d.%d or greater required." % (MINIMUM_PYQT_MAJOR_VER, MINIMUM_PYQT_MINOR_VER))
+                    else:
+                        log.info("OK, version %d.%d installed." % (maj_ver, min_ver))
+            del qt
 
 
-    log.info()
-    log.info(log.bold("Checking SIP version..."))
-
-    sip_ver = None
-    try:
-        import pyqtconfig
-    except ImportError:
-        pass
     else:
-        sip_ver = pyqtconfig.Configuration().sip_version_str 
 
-    if sip_ver is not None:
-        log.info("OK, Version %s installed" % sip_ver)
-    else:
-        num_errors += 1
-        log.error("SIP not installed or version not found.")
+        log.info()
+        log.info(log.bold("Checking PyQt 4.x version..."))
+
+        # PyQt 4
+        try:
+            import PyQt4
+        except ImportError:
+            num_errors += 1
+            log.error("NOT FOUND OR FAILED TO LOAD!")
+        else:
+            from PyQt4 import QtCore
+            log.info("OK, version %s installed." % QtCore.PYQT_VERSION_STR)
+
+
+#    log.info()
+#    log.info(log.bold("Checking SIP version..."))
+#
+#    sip_ver = None
+#    try:
+#        import pyqtconfig
+#    except ImportError:
+#        pass
+#    else:
+#        sip_ver = pyqtconfig.Configuration().sip_version_str
+#
+#    if sip_ver is not None:
+#        log.info("OK, Version %s installed" % sip_ver)
+#    else:
+#        num_errors += 1
+#        log.error("SIP not installed or version not found.")
 
     log.info()
     log.info(log.bold("Checking for CUPS..."))
@@ -315,10 +336,10 @@ try:
             log.error("Version: (Not available. CUPS may not be installed or not running.)")
             cups_ok = False
             num_errors += 1
-        
+
     if cups_ok:
         cups_conf = '/etc/cups/cupsd.conf'
-        
+
         try:
             f = file(cups_conf, 'r')
         except (IOError, OSError):
@@ -329,30 +350,30 @@ try:
                 if m is not None:
                     level = m.group(1).lower()
                     log.info("error_log is set to level: %s" % level)
-                    
-                    if level not in ('debug', 'debug2'):
-                        log.note("For troubleshooting printing issues, it is best to have the CUPS 'LogLevel'")
-                        log.note("set to 'debug'. To set the LogLevel to debug, edit the file %s (as root)," % cups_conf)
-                        log.note("and change the line near the top of the file that begins with 'LogLevel' to read:")
-                        log.note("LogLevel debug")
-                        log.note("Save the file and then restart CUPS (see your OS/distro docs on how to restart CUPS).")
-                        log.note("Now, when you print, helpful debug information will be saved to the file:")
-                        log.note("/var/log/cups/error_log")
-                        log.note("You can monitor this file by running this command in a console/shell:")
-                        log.note("tail -f /var/log/cups/error_log")
-                    
+
+                    #if level not in ('debug', 'debug2'):
+                        #log.note("For troubleshooting printing issues, it is best to have the CUPS 'LogLevel'")
+                        #log.note("set to 'debug'. To set the LogLevel to debug, edit the file %s (as root)," % cups_conf)
+                        #log.note("and change the line near the top of the file that begins with 'LogLevel' to read:")
+                        #log.note("LogLevel debug")
+                        #log.note("Save the file and then restart CUPS (see your OS/distro docs on how to restart CUPS).")
+                        #log.note("Now, when you print, helpful debug information will be saved to the file:")
+                        #log.note("/var/log/cups/error_log")
+                        #log.note("You can monitor this file by running this command in a console/shell:")
+                        #log.note("tail -f /var/log/cups/error_log")
+
                     break
-                    
+
 
     log.info()
 
     log.info(log.bold("Checking for dbus/python-dbus..."))
-    
+
     if dcheck.check_ps(['dbus-daemon']):
         log.info("dbus daemon is running.")
     else:
         log.warn("dbus daemon is not running.")
-        
+
     try:
         import dbus
         try:
@@ -364,18 +385,18 @@ try:
                 log.warn("python-dbus imported OK, but unknown version.")
     except ImportError:
         log.warn("python-dbus not installed.")
-    
+
     log.info()
-    
+
 
     if time_flag == DEPENDENCY_RUN_AND_COMPILE_TIME:
         tui.header("COMPILE AND RUNTIME DEPENDENCIES")
         log.note("To check for compile-time only dependencies, re-run hp-check with the -c parameter (ie, hp-check -c).")
         log.note("To check for run-time only dependencies, re-run hp-check with the -r parameter (ie, hp-check -r).")
-    
+
     elif time_flag == DEPENDENCY_COMPILE_TIME:
         tui.header("COMPILE TIME DEPENDENCIES")
-    
+
     elif time_flag == DEPENDENCY_RUN_TIME:
         tui.header("RUNTIME DEPENDENCIES")
 
@@ -384,10 +405,14 @@ try:
     dd = core.dependencies.keys()
     dd.sort()
     for d in dd:
+        if (d == 'pyqt' and core.ui_toolkit != 'qt3') or \
+           (d == 'pyqt4' and core.ui_toolkit != 'qt4'):
+            continue
+
         log.debug("***")
 
         if time_flag == DEPENDENCY_RUN_AND_COMPILE_TIME or time_flag == core.dependencies[d][4]:
-                
+
             log.info(log.bold("Checking for dependency: %s..." % core.dependencies[d][2]))
 
             if core.have_dependencies[d]:
@@ -399,10 +424,10 @@ try:
                     s = ''
                 elif core.dependencies[d][4] == DEPENDENCY_COMPILE_TIME:
                     s = '/COMPILE TIME ONLY'
-                    
+
                 elif core.dependencies[d][4] == DEPENDENCY_RUN_TIME:
                     s = '/RUNTIME ONLY'
-                
+
                 if core.dependencies[d][0]:
                     log.error("NOT FOUND! This is a REQUIRED%s dependency. Please make sure that this dependency is installed before installing or running HPLIP." % s)
                 else:
@@ -436,68 +461,81 @@ try:
 
 
             log.info()
-    
+
     if time_flag in (DEPENDENCY_RUN_TIME, DEPENDENCY_RUN_AND_COMPILE_TIME):
         tui.header("HPLIP INSTALLATION")
 
-        scanning_enabled = utils.to_bool(sys_cfg.configure.get("scanner-build", False))
+        scanning_enabled = utils.to_bool(sys_conf.get('configure', 'scanner-build', '0'))
 
         log.info()
         log.info(log.bold("Currently installed HPLIP version..."))
-        v = sys_cfg.hplip.version
-        home = sys_cfg.dirs.home
+        v = sys_conf.get('hplip', 'version')
+        home = sys_conf.get('dirs', 'home')
 
         if v:
             log.info("HPLIP %s currently installed in '%s'." % (v, home))
 
             log.info()
             log.info(log.bold("Current contents of '/etc/hp/hplip.conf' file:"))
-            output = file('/etc/hp/hplip.conf', 'r').read()
-            log.info(output)
+            try:
+                output = file('/etc/hp/hplip.conf', 'r').read()
+            except (IOError, OSError), e:
+                log.error("Could not access file: %s" % e.strerror)
+            else:
+                log.info(output)
+
+            log.info()
+            log.info(log.bold("Current contents of '~/.hplip/hplip.conf' file:"))
+            try:
+                output = file(os.path.expanduser('~/.hplip/hplip.conf'), 'r').read()
+            except (IOError, OSError), e:
+                log.error("Could not access file: %s" % e.strerror)
+            else:
+                log.info(output)
 
         else:
-            log.info("Not found.")  
+            log.info("Not found.")
+
 
-        
         if device_avail:
             if prop.par_build:
                 tui.header("DISCOVERED PARALLEL DEVICES")
-                
+
                 devices = device.probeDevices(['par'])
-                
+
                 if devices:
                     f = tui.Formatter()
                     f.header = ("Device URI", "Model")
-                    
+
                     for d, dd in devices.items():
                         f.add((d, dd[0]))
 
                     f.output()
-                
+
                 else:
                     log.info("No devices found.")
-                    
+
                     if not core.have_dependencies['ppdev']:
                         log.error("'ppdev' kernel module not loaded.")
-            
+
             if prop.usb_build:
-                tui.header("DISCOVERED USB DEVICES")                
-                
+                tui.header("DISCOVERED USB DEVICES")
+
                 devices = device.probeDevices(['usb'])
-                
+
                 if devices:
                     f = tui.Formatter()
                     f.header = ("Device URI", "Model")
-                    
+
                     for d, dd in devices.items():
                         f.add((d, dd[0]))
 
                     f.output()
-                    
+
                 else:
                     log.info("No devices found.")
-        
-        
+
+
         tui.header("INSTALLED CUPS PRINTER QUEUES")
 
         lpstat_pat = re.compile(r"""^device for (.*): (.*)""", re.IGNORECASE)
@@ -566,11 +604,11 @@ try:
                     status, output = utils.run('lpstat -p%s' % printer_name)
                     log.info("Printer status: %s" % output.replace("\n", ""))
 
-                    if back_end == 'hpfax' and desc != 'HP Fax':
+                    if back_end == 'hpfax' and not 'HP Fax' in desc:
                         num_errors += 1
                         log.error("Incorrect PPD file for fax queue '%s'. Fax queues must use 'HP-Fax-hplip.ppd'." % printer_name)
 
-                    elif back_end == 'hp' and desc == 'HP Fax':
+                    elif back_end == 'hp' and 'HP Fax' in desc:
                         num_errors += 1
                         log.error("Incorrect PPD file for a print queue '%s'. Print queues must not use 'HP-Fax-hplip.ppd'." % printer_name)
 
@@ -589,25 +627,20 @@ try:
 
                         plugin = d.mq.get('plugin', PLUGIN_NONE)
                         if plugin in (PLUGIN_REQUIRED, PLUGIN_OPTIONAL):
-                            home = sys_cfg.dirs.home or os.path.realpath(os.path.normpath(os.getcwd()))
-                            
-                            log.debug("home=%s" % home)
 
-                            model = model.lower()
-
-                            if os.path.exists(os.path.join(home, "data", "plugins", "%s.plugin" % model)):
+                            if core.check_for_plugin():
                                 if plugin == PLUGIN_REQUIRED:
                                     log.info("Required plug-in status: Installed")
                                 else:
                                     log.info("Optional plug-in status: Installed")
                             else:
                                 num_errors += 1
-                                
+
                                 if plugin == PLUGIN_REQUIRED:
                                     log.error("Required plug-in status: Not installed")
                                 else:
-                                    log.warn("Optional plug-in status: Not installed") 
-                                
+                                    log.warn("Optional plug-in status: Not installed")
+
 
                         if bus in ('par', 'usb'):
                             try:
@@ -631,7 +664,7 @@ try:
                             except Error:
                                 #log.error("Communication with device failed.")
                                 error_code = pml.ERROR_COMMAND_EXECUTION
-                            
+
                             if error_code > pml.ERROR_MAX_OK:
                                 #log.error("Communication status: Failed")
                                 num_errors += 1
@@ -708,7 +741,7 @@ try:
             num_errors += 1
             log.error("NOT FOUND OR FAILED TO LOAD! Please reinstall HPLIP and check for the proper installation of hpmudext.")
         else:
-            log.info("OK, found.")        
+            log.info("OK, found.")
 
         if scanning_enabled:
             log.info()
@@ -719,7 +752,7 @@ try:
                 num_errors += 1
                 log.error("NOT FOUND OR FAILED TO LOAD! Please reinstall HPLIP and check for the proper installation of scanext.")
             else:
-                log.info("OK, found.")        
+                log.info("OK, found.")
 
                 log.info()
 
@@ -796,7 +829,7 @@ try:
 
 except KeyboardInterrupt:
     log.error("User exit")
-    
+
 log.info()
 log.info("Done.")
 
