@@ -273,10 +273,9 @@ enum HPMUD_RESULT __attribute__ ((visibility ("hidden"))) jd_channel_open(mud_de
    else
    {
       if ((stat = (pd->channel[index].vf.open)(&pd->channel[index])) != HPMUD_R_OK)  /* call transport specific open */
-      {
          del_channel(pd, &pd->channel[index]);   /* open failed, cleanup */
-      }
-      *cd = index;
+      else
+         *cd = index;
    }
 
    pthread_mutex_unlock(&pd->mutex);
@@ -417,6 +416,48 @@ enum HPMUD_RESULT __attribute__ ((visibility ("hidden"))) jd_s_channel_open(mud_
             goto bugout;  
          }
          break; 
+      case HPMUD_SOAPSCAN_CHANNEL:
+         port = 8289;
+         pin.sin_port = htons(port);
+         if ((pc->socket = socket(AF_INET, SOCK_STREAM, 0)) == -1) 
+         {  
+            BUG("unable to open soap-scan port %d: %m %s\n", port, pd->uri);  
+            goto bugout;  
+         }  
+         if (connect(pc->socket, (struct sockaddr *)&pin, sizeof(pin)) == -1) 
+         {  
+            BUG("unable to connect to soap-scan port %d: %m %s\n", port, pd->uri);  
+            goto bugout;  
+         }
+         break; 
+      case HPMUD_SOAPFAX_CHANNEL:
+         port = 8295;
+         pin.sin_port = htons(port);
+         if ((pc->socket = socket(AF_INET, SOCK_STREAM, 0)) == -1) 
+         {  
+            BUG("unable to open soap-fax port %d: %m %s\n", port, pd->uri);  
+            goto bugout;  
+         }  
+         if (connect(pc->socket, (struct sockaddr *)&pin, sizeof(pin)) == -1) 
+         {  
+            BUG("unable to connect to soap-fax port %d: %m %s\n", port, pd->uri);  
+            goto bugout;  
+         }
+         break; 
+      case HPMUD_MARVELL_SCAN_CHANNEL:
+         port = 8290;  /* same as ScanPort1[1] */
+         pin.sin_port = htons(port);
+         if ((pc->socket = socket(AF_INET, SOCK_STREAM, 0)) == -1) 
+         {  
+            BUG("unable to open marvell-scan port %d: %m %s\n", port, pd->uri);  
+            goto bugout;  
+         }  
+         if (connect(pc->socket, (struct sockaddr *)&pin, sizeof(pin)) == -1) 
+         {  
+            BUG("unable to connect to marvell-scan port %d: %m %s\n", port, pd->uri);  
+            goto bugout;  
+         }
+         break; 
       case HPMUD_PML_CHANNEL:
          /* Do nothing here, use GetPml/SetPml instead of ReadData/WriteData. */
          break;
