@@ -1,7 +1,7 @@
 /*****************************************************************************\
   context.cpp : Implimentation for the PrintContext class
 
-  Copyright (c) 1996 - 2001, Hewlett-Packard Co.
+  Copyright (c) 1996 - 2008, Hewlett-Packard Co.
   All rights reserved.
 
   Redistribution and use in source and binary forms, with or without
@@ -185,7 +185,9 @@ const PrintContext::PaperSizeMetrics PrintContext::PSM[MAX_PAPER_SIZE] =
 	{
 		(float)3.54, (float)8.07, (float)3.29, (float)7.445,  (float)0.125
 	},
-#endif
+
+#endif // APDK_EXTENDED_MEDIASIZE
+
     // PHOTO_5x7 = 5in x 7in = 127 mm x 177.8 mm
     {
         (float) 5.0, (float) 7.0, (float) 4.75, (float) 6.375, (float) 0.125
@@ -199,6 +201,22 @@ const PrintContext::PaperSizeMetrics PrintContext::PSM[MAX_PAPER_SIZE] =
         (float) 5.0, (float) 5.0, (float) 5.0, (float) 5.0, (float) 0.0
     }
 
+#ifdef APDK_EXTENDED_MEDIASIZE
+    // PHOTO_4x8 - 4x8 panorama photo
+    ,{
+        (float) 4.0, (float) 6.0, (float) 3.75, (float) 7.75, (float) 0.125
+    },
+
+    // PHOTO_4x12 - 4x12 panorama photo
+    {
+        (float) 4.0, (float) 12.0, (float) 3.75, (float) 11.75, (float) 0.125
+    },
+    // L - Japanese Card, 3.5 x 5 in
+    {
+        (float) 3.5, (float) 5.0, (float) 3.25, (float) 4.75, (float) 0.125
+    }
+#endif // APDK_EXTENDED_MEDIASIZE
+
 }; //PSM
 
 
@@ -1245,14 +1263,17 @@ DRIVER_ERROR PrintContext::SelectDevice
     {
         return UNSUPPORTED_PRINTER;
     }
-    thePrinter = pPFI->CreatePrinter(pSS, familyHandle);
+    thePrinter = pPFI->CreatePrinter (pSS, familyHandle);
+    if (thePrinter->constructor_error != NO_ERROR)
+    {
+        return thePrinter->constructor_error;
+    }
 
+    const char* model = pPFI->GetFamilyName (familyHandle);
 
-    const char* model = pPFI->GetFamilyName(familyHandle);
+    pSS->AdjustIO (thePrinter->IOMode, model);
 
-    pSS->AdjustIO(thePrinter->IOMode, model);
-
-    PAPER_SIZE ps = thePrinter->MandatoryPaperSize();
+    PAPER_SIZE ps = thePrinter->MandatoryPaperSize ();
     if (ps != UNSUPPORTED_SIZE)
     {
         if ((PSM[ps].fPhysicalPageX < PSM[thePaperSize].fPhysicalPageX))
@@ -2030,6 +2051,21 @@ void PrintContext::ResetIOMode (BOOL bDevID, BOOL bStatus)
         thePrinter->bCheckForCancelButton = bDevID;
     }
 }
-  
+
+DRIVER_ERROR PrintContext::SetPrinterHint (PRINTER_HINT eHint, int iValue)
+{
+    if (thePrinter)
+    {
+        return thePrinter->SetHint (eHint, iValue);
+    }
+    return NO_PRINTER_SELECTED;
+}
+
+DRIVER_ERROR PrintContext::SetMediaType (MEDIATYPE eMediaType)
+{
+    if (CurrentMode == NULL)
+        return NO_PRINTER_SELECTED;
+    return CurrentMode->SetMediaType (eMediaType);
+}
 
 APDK_END_NAMESPACE
