@@ -34,6 +34,18 @@ import os.path
 import os
 
 
+class RangeValidator(QValidator):
+    def __init__(self, parent=None, name=None):
+        QValidator.__init__(self, parent, name)
+
+    def validate(self, input, pos):
+        for x in unicode(input)[pos-1:]:
+            if x not in u'0123456789,- ':
+                return QValidator.Invalid, pos
+
+        return QValidator.Acceptable, pos
+
+
 class OptionComboBox(QComboBox):
     def __init__(self, rw, parent, name, group, option, choices, default, typ=cups.PPD_UI_PICKONE, other=None):
         QComboBox.__init__(self, rw, parent, name)
@@ -46,7 +58,7 @@ class OptionComboBox(QComboBox):
 
     def setDefaultPushbutton(self, pushbutton):
         self.pushbutton = pushbutton
-        
+
     def setOther(self, other):
         self.other = other
 
@@ -71,8 +83,8 @@ class OptionButtonGroup(QButtonGroup):
 
     def setDefaultPushbutton(self, pushbutton):
         self.pushbutton = pushbutton
-            
-        
+
+
 class DefaultPushButton(QPushButton):
     def __init__(self,  parent, name, group, option, choices, default, control, typ):
         QPushButton.__init__(self, parent, name)
@@ -82,7 +94,7 @@ class DefaultPushButton(QPushButton):
         self.control = control
         self.typ = typ
         self.choices = choices
-        
+
 
 
 class ScrollPrintSettingsView(ScrollView):
@@ -90,16 +102,16 @@ class ScrollPrintSettingsView(ScrollView):
 
     def __init__(self, service, parent=None, name=None, fl=0):
         ScrollView.__init__(self, service, parent, name, fl)
-        
+
         self.initJobStorage(True)
 
-    
-    
+
+
     def fillControls(self):
         QApplication.setOverrideCursor(QApplication.waitCursor)
-    
+
         ScrollView.fillControls(self)
-    
+
         self.loading = True
         cups.resetOptions()
         cups.openPPD(self.cur_printer)
@@ -109,64 +121,64 @@ class ScrollPrintSettingsView(ScrollView):
             if 1:
             #try:
                 current_options = dict(cups.getOptions())
-                
+
                 if not self.cur_device.device_type == DEVICE_TYPE_FAX:
                     self.addGroupHeading("basic", self.__tr("Basic"))
                     log.debug("Group: Basic")
-    
+
                     # Basic
                         # PageSize (in PPD section)
                         # orientation-requested
                         # sides
                         # outputorder
                         # Collate
-    
-                    
+
+
                     current = current_options.get('orientation-requested', '3')
-    
-                    self.addItem("basic", "orientation-requested", self.__tr("Page Orientation"), 
-                        cups.PPD_UI_PICKONE, current, 
-                        [('3', self.__tr('Portrait')), 
-                         ('4', self.__tr('Landscape')), 
-                         ('5', self.__tr('Reverse landscape')), 
+
+                    self.addItem("basic", "orientation-requested", self.__tr("Page Orientation"),
+                        cups.PPD_UI_PICKONE, current,
+                        [('3', self.__tr('Portrait')),
+                         ('4', self.__tr('Landscape')),
+                         ('5', self.__tr('Reverse landscape')),
                          ('6', self.__tr('Reverse portrait'))], '3')
-    
+
                     log.debug("Option: orientation-requested")
                     log.debug("Current value: %s" % current)
-    
+
                     duplexer = self.cur_device.dq.get('duplexer', 0)
                     log.debug("Duplexer = %d" % duplexer)
-    
+
                     if duplexer:
                         current = current_options.get('sides', 'one-sided')
-                        self.addItem("basic", "sides", 
-                            self.__tr("Duplex (Print on both sides of the page)"), 
-                            cups.PPD_UI_PICKONE, current, 
-                            [('one-sided',self.__tr('Single sided')), 
-                             ('two-sided-long-edge', self.__tr('Two sided (long edge)')), 
+                        self.addItem("basic", "sides",
+                            self.__tr("Duplex (Print on both sides of the page)"),
+                            cups.PPD_UI_PICKONE, current,
+                            [('one-sided',self.__tr('Single sided')),
+                             ('two-sided-long-edge', self.__tr('Two sided (long edge)')),
                              ('two-sided-short-edge', self.__tr('Two sided (short edge)'))], 'one-sided')
-    
+
                         log.debug("Option: sides")
                         log.debug("Current value: %s" % current)
-    
+
                     current = current_options.get('outputorder', 'normal')
-    
-                    self.addItem("basic", "outputorder", 
-                        self.__tr("Output Order (Print last page first)"), 
-                        cups.PPD_UI_PICKONE, current, 
-                        [('normal', self.__tr('Normal (Print first page first)')), 
+
+                    self.addItem("basic", "outputorder",
+                        self.__tr("Output Order (Print last page first)"),
+                        cups.PPD_UI_PICKONE, current,
+                        [('normal', self.__tr('Normal (Print first page first)')),
                          ('reverse', self.__tr('Reversed (Print last page first)'))], 'normal')
-    
+
                     log.debug("Option: outputorder")
                     log.debug("Current value: %s" % current)
-    
-                    current = int(utils.to_bool(current_options.get('Collate', '0')))
-    
-                    self.addItem("basic", "Collate", 
-                        self.__tr("Collate (Group together multiple copies)"), 
-                        cups.PPD_UI_BOOLEAN, current, 
+
+                    current = utils.to_bool(current_options.get('Collate', '0'))
+
+                    self.addItem("basic", "Collate",
+                        self.__tr("Collate (Group together multiple copies)"),
+                        cups.PPD_UI_BOOLEAN, current,
                         [], 0)
-    
+
                     log.debug("Option: Collate")
                     log.debug("Current value: %s" % current)
 
@@ -174,22 +186,22 @@ class ScrollPrintSettingsView(ScrollView):
 
                 for g in groups:
                     log.debug("Group: %s" % repr(g))
-                    
+
                     if 'jobretention' in g.lower():
                         log.debug("HPJobRetention skipped.")
-                        continue                    
-                    
-                    text, num_subgroups = cups.getGroup(g) 
+                        continue
+
+                    text, num_subgroups = cups.getGroup(g)
                     read_only = 'install' in g.lower()
-                    
+
                     try:
                         text = text.decode('utf-8')
                     except UnicodeDecodeError:
                         pass
-                    
+
                     if g.lower() == 'printoutmode':
                         text = self.__tr("Quality")
-                    
+
                     self.addGroupHeading(g, text, read_only)
 
                     log.debug("  Text: %s" % repr(text))
@@ -203,8 +215,8 @@ class ScrollPrintSettingsView(ScrollView):
                         if 'pageregion' in o.lower():
                             log.debug("Page Region skipped.")
                             continue
-                            
-                        
+
+
 
                         option_text, defchoice, conflicted, ui  = cups.getOption(g, o)
 
@@ -212,10 +224,10 @@ class ScrollPrintSettingsView(ScrollView):
                             option_text = option_text.decode('utf-8')
                         except UnicodeDecodeError:
                             pass
-                        
+
                         if o.lower() == 'quality':
                             option_text = self.__tr("Quality")
-                                                    
+
                         log.debug("    Text: %s" % repr(option_text))
                         log.debug("    Defchoice: %s" % repr(defchoice))
 
@@ -237,7 +249,7 @@ class ScrollPrintSettingsView(ScrollView):
                                 choice_text = choice_text.decode('utf-8')
                             except UnicodeDecodeError:
                                 pass
-                            
+
                             log.debug("      Text: %s" % repr(choice_text))
 
                             if marked:
@@ -247,16 +259,16 @@ class ScrollPrintSettingsView(ScrollView):
 
 
                         self.addItem(g, o, option_text, ui, value, choice_data, defchoice, read_only)
-                        
+
 ##                        if 'pagesize' in o.lower(): # and 'custom' in c.lower():
 ##                            current = 0.0
 ##                            width_widget = self.addItem("custom", "custom-width", self.__tr("Custom Paper Width"), cups.UI_UNITS_SPINNER,
-##                                current, (0.0, 0.0), 0.0) 
-##                            
+##                                current, (0.0, 0.0), 0.0)
+##
 ##                            current = 0.0
 ##                            height_widget = self.addItem("custom", "custom-height", self.__tr("Custom Paper Height"), cups.UI_UNITS_SPINNER,
-##                                current, (0.0, 0.0), 0.0) 
-##                                
+##                                current, (0.0, 0.0), 0.0)
+##
 ##                            if value.lower() == 'custom':
 ##                                pass
 
@@ -265,33 +277,33 @@ class ScrollPrintSettingsView(ScrollView):
                     # number-up-layout
                     # page-border
 
-                self.addGroupHeading("nup", 
+                self.addGroupHeading("nup",
                     self.__tr("N-Up (Multiple document pages per printed page)"))
 
                 log.debug("Group: N-Up")
 
                 current = current_options.get('number-up', '1')
 
-                self.addItem("nup", "number-up", self.__tr("Pages per Sheet"), 
-                    cups.PPD_UI_PICKONE, current, 
-                    [('1', self.__tr('1 sheet per page')), 
-                     ('2', self.__tr('2 sheets per page')), 
-                     ('4', self.__tr('4 sheets per page'))], '1')
+                self.addItem("nup", "number-up", self.__tr("Pages per Sheet"),
+                    cups.PPD_UI_PICKONE, current,
+                    [('1', self.__tr('1 page per sheet')),
+                     ('2', self.__tr('2 pages per sheet')),
+                     ('4', self.__tr('4 pages per sheet'))], '1')
 
                 log.debug("  Option: number-up")
                 log.debug("  Current value: %s" % current)
 
                 current = current_options.get('number-up-layout', 'lrtb')
 
-                self.addItem("nup", "number-up-layout", self.__tr("Layout"), 
-                    cups.PPD_UI_PICKONE, current, 
-                    [('btlr', self.__tr('Bottom to top, left to right')), 
-                     ('btrl', self.__tr('Bottom to top, right to left')), 
-                     ('lrbt', self.__tr('Left to right, bottom to top')), 
+                self.addItem("nup", "number-up-layout", self.__tr("Layout"),
+                    cups.PPD_UI_PICKONE, current,
+                    [('btlr', self.__tr('Bottom to top, left to right')),
+                     ('btrl', self.__tr('Bottom to top, right to left')),
+                     ('lrbt', self.__tr('Left to right, bottom to top')),
                      ('lrtb', self.__tr('Left to right, top to bottom')),
-                     ('rlbt', self.__tr('Right to left, bottom to top')), 
+                     ('rlbt', self.__tr('Right to left, bottom to top')),
                      ('rltb', self.__tr('Right to left, top to bottom')),
-                     ('tblr', self.__tr('Top to bottom, left to right')), 
+                     ('tblr', self.__tr('Top to bottom, left to right')),
                      ('tbrl', self.__tr('Top to bottom, right to left')) ], 'lrtb')
 
                 log.debug("  Option: number-up-layout")
@@ -299,13 +311,13 @@ class ScrollPrintSettingsView(ScrollView):
 
                 current = current_options.get('page-border', 'none')
 
-                self.addItem("nup", "page-border", 
-                    self.__tr("Printed Border Around Each Page"), 
+                self.addItem("nup", "page-border",
+                    self.__tr("Printed Border Around Each Page"),
                     cups.PPD_UI_PICKONE, current,
-                    [('double', self.__tr("Two thin borders")), 
+                    [('double', self.__tr("Two thin borders")),
                      ("double-thick", self.__tr("Two thick borders")),
-                     ("none", self.__tr("No border")), 
-                     ("single", self.__tr("One thin border")), 
+                     ("none", self.__tr("No border")),
+                     ("single", self.__tr("One thin border")),
                      ("single-thick", self.__tr("One thick border"))], 'none')
 
                 log.debug("  Option: page-border")
@@ -317,23 +329,23 @@ class ScrollPrintSettingsView(ScrollView):
 
                 if not self.cur_device.device_type == DEVICE_TYPE_FAX:
                     self.addGroupHeading("adjustment", self.__tr("Printout Appearance"))
-    
+
                     current = int(current_options.get('brightness', 100))
-    
+
                     log.debug("  Option: brightness")
                     log.debug("  Current value: %s" % current)
-    
+
                     self.addItem("adjustment", "brightness", self.__tr("Brightness"),
                         cups.UI_SPINNER, current, (0, 200), 100, suffix=" %")
-    
+
                     current = int(current_options.get('gamma', 1000))
-    
+
                     log.debug("  Option: gamma")
                     log.debug("  Current value: %s" % current)
-    
+
                     self.addItem("adjustment", "gamma", self.__tr("Gamma"), cups.UI_SPINNER, current,
                         (1, 10000), 1000)
-    
+
                 # Margins (pts)
                     # page-left
                     # page-right
@@ -357,19 +369,19 @@ class ScrollPrintSettingsView(ScrollView):
 ##                    log.debug("  Option: page-top")
 ##                    log.debug("  Current value: %s" % current_top)
 ##
-##                    self.addItem("margins", "page-top", self.__tr("Top margin"), 
+##                    self.addItem("margins", "page-top", self.__tr("Top margin"),
 ##                        cups.UI_UNITS_SPINNER, current_top,
 ##                        (0, page_len), top)
 ##
-##                    self.addItem("margins", "page-bottom", self.__tr("Bottom margin"), 
+##                    self.addItem("margins", "page-bottom", self.__tr("Bottom margin"),
 ##                        cups.UI_UNITS_SPINNER, current_bottom,
 ##                        (0, page_len), bottom)
 ##
-##                    self.addItem("margins", "page-left", self.__tr("Right margin"), 
+##                    self.addItem("margins", "page-left", self.__tr("Right margin"),
 ##                        cups.UI_UNITS_SPINNER, current_left,
 ##                        (0, page_width), left)
 ##
-##                    self.addItem("margins", "page-right", self.__tr("Left margin"), 
+##                    self.addItem("margins", "page-right", self.__tr("Left margin"),
 ##                        cups.UI_UNITS_SPINNER, current_right,
 ##                        (0, page_width), right)
 
@@ -381,26 +393,26 @@ class ScrollPrintSettingsView(ScrollView):
 
                 self.addGroupHeading("image", self.__tr("Image Printing"))
 
-                current = current_options.get('fitplot', 'false')
-                
-                self.addItem("image", "fitplot", 
-                    self.__tr("Fit to Page"), 
-                    cups.PPD_UI_BOOLEAN, current, 
+                current = utils.to_bool(current_options.get('fitplot', 'false'))
+
+                self.addItem("image", "fitplot",
+                    self.__tr("Fit to Page"),
+                    cups.PPD_UI_BOOLEAN, current,
                     [], 0)
 
-                
+
                 current = current_options.get('position', 'center')
 
-                self.addItem("image", "position", self.__tr("Position on Page"), 
-                    cups.PPD_UI_PICKONE, current, 
-                    [('center', self.__tr('Centered')), 
-                     ('top', self.__tr('Top')), 
-                     ('left', self.__tr('Left')), 
-                     ('right', self.__tr('Right')), 
-                     ('top-left', self.__tr('Top left')), 
-                     ('top-right', self.__tr('Top right')), 
+                self.addItem("image", "position", self.__tr("Position on Page"),
+                    cups.PPD_UI_PICKONE, current,
+                    [('center', self.__tr('Centered')),
+                     ('top', self.__tr('Top')),
+                     ('left', self.__tr('Left')),
+                     ('right', self.__tr('Right')),
+                     ('top-left', self.__tr('Top left')),
+                     ('top-right', self.__tr('Top right')),
                      ('bottom', self.__tr('Bottom')),
-                     ('bottom-left', self.__tr('Bottom left')), 
+                     ('bottom-left', self.__tr('Bottom left')),
                      ('bottom-right', self.__tr('Bottom right'))], 'center')
 
                 log.debug("  Option: position")
@@ -408,19 +420,19 @@ class ScrollPrintSettingsView(ScrollView):
 
                 if not self.cur_device.device_type == DEVICE_TYPE_FAX:
                     current = int(current_options.get('saturation', 100))
-    
+
                     log.debug("  Option: saturation")
                     log.debug("  Current value: %s" % current)
-    
-                    self.addItem("image", "saturation", self.__tr("Saturation"), 
+
+                    self.addItem("image", "saturation", self.__tr("Saturation"),
                         cups.UI_SPINNER, current, (0, 200), 100, suffix=" %")
-    
+
                     current = int(current_options.get('hue', 0))
-    
+
                     log.debug("  Option: hue")
                     log.debug("  Current value: %s" % current)
-    
-                    self.addItem("image", "hue", self.__tr("Hue (color shift/rotation)"), 
+
+                    self.addItem("image", "hue", self.__tr("Hue (color shift/rotation)"),
                         cups.UI_SPINNER, current,
                         (-100, 100), 0)
 
@@ -429,8 +441,8 @@ class ScrollPrintSettingsView(ScrollView):
                 log.debug("  Option: natural-scaling")
                 log.debug("  Current value: %s" % current)
 
-                self.addItem("image", "natural-scaling", 
-                    self.__tr('"Natural" Scaling (relative to image)'), 
+                self.addItem("image", "natural-scaling",
+                    self.__tr('"Natural" Scaling (relative to image)'),
                     cups.UI_SPINNER, current, (1, 800), 100, suffix=" %")
 
                 current = int(current_options.get('scaling', 100))
@@ -438,7 +450,7 @@ class ScrollPrintSettingsView(ScrollView):
                 log.debug("  Option: scaling")
                 log.debug("  Current value: %s" % current)
 
-                self.addItem("image", "scaling", self.__tr("Scaling (relative to page)"), 
+                self.addItem("image", "scaling", self.__tr("Scaling (relative to page)"),
                     cups.UI_SPINNER, current,
                     (1, 800), 100, suffix=" %")
 
@@ -451,9 +463,9 @@ class ScrollPrintSettingsView(ScrollView):
 
                 log.debug("Group: Misc")
 
-                current = int(utils.to_bool(current_options.get('prettyprint', '0')))
+                current = utils.to_bool(current_options.get('prettyprint', '0'))
 
-                self.addItem("misc", "prettyprint", 
+                self.addItem("misc", "prettyprint",
                     self.__tr('"Pretty Print" Text Documents (Add headers and formatting)'),
                     cups.PPD_UI_BOOLEAN, current, [], 0)
 
@@ -462,47 +474,47 @@ class ScrollPrintSettingsView(ScrollView):
 
                 if not self.cur_device.device_type == DEVICE_TYPE_FAX:
                     current = current_options.get('job-sheets', 'none').split(',')
-                    
+
                     try:
                         start = current[0]
                     except IndexError:
                         start = 'none'
-                        
+
                     try:
                         end = current[1]
                     except IndexError:
                         end = 'none'
-                    
+
                     # TODO: Look for locally installed banner pages beyond the default CUPS ones?
-                    self.addItem("misc", "job-sheets", self.__tr("Banner Pages"), cups.UI_BANNER_JOB_SHEETS, 
-                        (start, end), 
-                        [("none", self.__tr("No banner page")), 
-                         ('classified', self.__tr("Classified")), 
+                    self.addItem("misc", "job-sheets", self.__tr("Banner Pages"), cups.UI_BANNER_JOB_SHEETS,
+                        (start, end),
+                        [("none", self.__tr("No banner page")),
+                         ('classified', self.__tr("Classified")),
                          ('confidential', self.__tr("Confidential")),
-                         ('secret', self.__tr("Secret")), 
-                         ('standard', self.__tr("Standard")), 
-                         ('topsecret', self.__tr("Top secret")), 
+                         ('secret', self.__tr("Secret")),
+                         ('standard', self.__tr("Standard")),
+                         ('topsecret', self.__tr("Top secret")),
                          ('unclassified', self.__tr("Unclassified"))], ('none', 'none'))
-                    
+
                     log.debug("  Option: job-sheets")
                     log.debug("  Current value: %s,%s" % (start, end))
 
-                current = int(utils.to_bool(current_options.get('mirror', '0')))
+                current = utils.to_bool(current_options.get('mirror', '0'))
 
                 self.addItem("misc", "mirror", self.__tr('Mirror Printing'),
                     cups.PPD_UI_BOOLEAN, current, [], 0)
 
                 log.debug("  Option: mirror")
                 log.debug("  Current value: %s" % current)
-                
+
                 self.job_storage_avail = self.cur_device.mq['job-storage'] == JOB_STORAGE_ENABLE
-            
+
                 #print current_options
-                
+
                 if self.job_storage_avail:
                     self.addGroupHeading("jobstorage", self.__tr("Job Storage and Secure Printing"))
                     self.addJobStorage(current_options)
-                    
+
 
             #except Exception, e:
                 #log.exception()
@@ -512,25 +524,25 @@ class ScrollPrintSettingsView(ScrollView):
             cups.closePPD()
             self.loading = False
             QApplication.restoreOverrideCursor()
-            
+
 
     def optionComboBox_activated(self, a):
         a = unicode(a)
         sender = self.sender()
         choice = None
-        
+
         if sender.typ == cups.UI_BANNER_JOB_SHEETS:
             start, end = None, None
             for c, t in sender.choices:
                 if t == a:
                     start = c
                     break
-                    
+
             for c, t in sender.other.choices:
                 if t == sender.other.currentText():
                     end = c
                     break
-                
+
             if sender.option == 'end':
                 start, end = end, start
 
@@ -542,35 +554,35 @@ class ScrollPrintSettingsView(ScrollView):
                     sender.pushbutton.setEnabled(False)
             else:
                 sender.pushbutton.setEnabled(True)
-                
+
                 if start is not None and \
                     end is not None:
-                    
+
                     self.setPrinterOption('job-sheets', ','.join([start, end]))
-                
+
         else:
             choice = None
             for c, t in sender.choices:
                 if t == a:
                     choice = c
                     break
-    
+
             if choice is not None and choice.lower() == sender.default.lower():
                 self.removePrinterOption(sender.option)
                 sender.pushbutton.setEnabled(False)
             else:
                 sender.pushbutton.setEnabled(True)
-    
+
                 if choice is not None:
                     self.setPrinterOption(sender.option, choice)
-                    
+
             self.linkPrintoutModeAndQuality(sender.option, choice)
 
-    
+
     def linkPrintoutModeAndQuality(self, option, choice):
         if option.lower() == 'quality' and \
             choice is not None:
-            
+
             try:
                 c = self.items['o:PrintoutMode'].control
             except KeyError:
@@ -583,26 +595,26 @@ class ScrollPrintSettingsView(ScrollView):
                         c.setEnabled(True)
                         QToolTip.remove(c)
                         a = unicode(c.currentText())
-                        
+
                         # determine printoutmode default button state
                         link_choice = None
                         for x, t in c.choices:
                             if t == a:
                                 link_choice = x
                                 break
-                
+
                         if link_choice is not None and \
                             link_choice.lower() == c.default.lower():
-                            
+
                             c.pushbutton.setEnabled(False)
                         else:
                             c.pushbutton.setEnabled(True)
-                    
+
                     else: # fromprintoutmode not selected, disable printoutmode
                         c.setEnabled(False)
                         QToolTip.add(c, self.__tr("""Set Quality to "Controlled by 'Printout Mode'" to enable."""))
                         c.pushbutton.setEnabled(False)
-                            
+
 
 
     def optionSpinBox_valueChanged(self, i):
@@ -615,7 +627,7 @@ class ScrollPrintSettingsView(ScrollView):
             sender.pushbutton.setEnabled(True)
             self.setPrinterOption(sender.option, str(i))
 
-            
+
     def optionButtonGroup_clicked(self, b):
         sender = self.sender()
         b = int(b)
@@ -630,7 +642,7 @@ class ScrollPrintSettingsView(ScrollView):
                 self.setPrinterOption(sender.option, "true")
             else:
                 self.setPrinterOption(sender.option, "false")
-        
+
 
 
     def defaultPushButton_clicked(self):
@@ -642,7 +654,7 @@ class ScrollPrintSettingsView(ScrollView):
                 sender.control.setButton(1)
             else:
                 sender.control.setButton(0)
-                
+
             self.removePrinterOption(sender.option)
 
         elif sender.typ == cups.PPD_UI_PICKONE:
@@ -657,32 +669,32 @@ class ScrollPrintSettingsView(ScrollView):
             if choice is not None:
                 self.removePrinterOption(sender.option)
                 sender.control.setCurrentText(text)
-                
+
                 self.linkPrintoutModeAndQuality(sender.option, choice)
 
         elif sender.typ == cups.UI_SPINNER:
             sender.control.setValue(sender.default)
             self.removePrinterOption(sender.option)
-            
+
         elif sender.typ == cups.UI_BANNER_JOB_SHEETS:
             start, end, start_text, end_text = None, None, None, None
             for c, t in sender.choices:
                 if c == sender.default[0]:
                     start = c
                     start_text = t
-                
+
                 if c == sender.default[1]:
                     end = c
                     end_text = t
-                    
+
             if start is not None:
                 sender.control[0].setCurrentText(start_text)
-                
+
             if end is not None:
                 sender.control[1].setCurrentText(end_text)
-                
+
             self.removePrinterOption('job-sheets')
-                
+
 
     def setPrinterOption(self, option, value):
         cups.openPPD(self.cur_printer)
@@ -711,10 +723,10 @@ class ScrollPrintSettingsView(ScrollView):
             layout = QGridLayout(widget, 1, 1, 5, 10, "layout")
             default = int(utils.to_bool(str(default)))
             value = int(utils.to_bool(str(value)))
-    
+
             textLabel1 = QLabel(widget, "textLabel1")
             layout.addWidget(textLabel1, 0, 0)
-    
+
             buttonGroup = OptionButtonGroup(widget, "buttonGroup", group, option, default)
             buttonGroup.setLineWidth(0)
             buttonGroup.setColumnLayout(0,Qt.Vertical)
@@ -722,34 +734,34 @@ class ScrollPrintSettingsView(ScrollView):
             buttonGroup.layout().setMargin(5)
             buttonGroupLayout = QHBoxLayout(buttonGroup.layout())
             buttonGroupLayout.setAlignment(Qt.AlignTop)
-    
-            defaultPushButton = DefaultPushButton(widget,"defaultPushButton", group, option, 
-                choices, default, buttonGroup, typ)    
-    
+
+            defaultPushButton = DefaultPushButton(widget,"defaultPushButton", group, option,
+                choices, default, buttonGroup, typ)
+
             buttonGroup.setDefaultPushbutton(defaultPushButton)
-            
+
             layout.addWidget(defaultPushButton, 0, 3)
-            
+
             spacer1 = QSpacerItem(20, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
             layout.addItem(spacer1, 0, 1)
-            
+
             onRadioButton = QRadioButton(buttonGroup,"onRadioButton")
             buttonGroup.insert(onRadioButton, 1)
             buttonGroupLayout.addWidget(onRadioButton)
-    
+
             offRadioButton = QRadioButton(buttonGroup,"offRadioButton")
             buttonGroup.insert(offRadioButton, 0)
             buttonGroupLayout.addWidget(offRadioButton)
-    
+
             layout.addWidget(buttonGroup, 0, 2)
-            
+
             textLabel1.setText(text)
             onRadioButton.setText(self.__tr("On"))
             offRadioButton.setText(self.__tr("Off"))
-            
+
             if value == default:
                 defaultPushButton.setEnabled(False)
-                
+
             self.connect(defaultPushButton, SIGNAL("clicked()"), self.defaultPushButton_clicked)
             self.connect(buttonGroup, SIGNAL("clicked(int)"), self.optionButtonGroup_clicked)
 
@@ -761,14 +773,14 @@ class ScrollPrintSettingsView(ScrollView):
                 buttonGroup.setButton(1)
             else:
                 buttonGroup.setButton(0)
-            
+
             if read_only:
                 onRadioButton.setEnabled(False)
                 offRadioButton.setEnabled(False)
                 defaultPushButton.setEnabled(False)
             else:
                 QToolTip.add(defaultPushButton, self.__tr('Set to default value of "%1".').arg(x))
-            
+
             defaultPushButton.setText("Default")
 
         elif typ == cups.PPD_UI_PICKONE: # Combo box widget
@@ -785,7 +797,7 @@ class ScrollPrintSettingsView(ScrollView):
             optionComboBox = OptionComboBox(0, widget, "optionComboBox", group, option, choices, default)
             layout1.addWidget(optionComboBox)
 
-            defaultPushButton = DefaultPushButton(widget,"defaultPushButton", group, option, 
+            defaultPushButton = DefaultPushButton(widget,"defaultPushButton", group, option,
                 choices, default, optionComboBox, typ)
 
             optionComboBox.setDefaultPushbutton(defaultPushButton)
@@ -811,7 +823,7 @@ class ScrollPrintSettingsView(ScrollView):
 
             if value is not None and value.lower() == default.lower():
                 defaultPushButton.setEnabled(False)
-                
+
             self.linkPrintoutModeAndQuality(option, value)
 
             if read_only:
@@ -822,7 +834,7 @@ class ScrollPrintSettingsView(ScrollView):
 
             self.connect(defaultPushButton, SIGNAL("clicked()"), self.defaultPushButton_clicked)
             self.connect(optionComboBox, SIGNAL("activated(const QString&)"), self.optionComboBox_activated)
-            
+
             control = optionComboBox
 
         elif typ == cups.UI_SPINNER: # Spinner widget
@@ -839,7 +851,7 @@ class ScrollPrintSettingsView(ScrollView):
             optionSpinBox = OptionSpinBox(widget,"optionSpinBox", group, option, default)
             layout1.addWidget(optionSpinBox)
 
-            defaultPushButton = DefaultPushButton(widget, "defaultPushButton", group, option, choices, 
+            defaultPushButton = DefaultPushButton(widget, "defaultPushButton", group, option, choices,
                 default, optionSpinBox, typ)
 
             optionSpinBox.setDefaultPushbutton(defaultPushButton)
@@ -867,49 +879,49 @@ class ScrollPrintSettingsView(ScrollView):
                 self.optionSpinBox.setEnabled(False)
                 self.defaultPushButton.setEnabled()
             else:
-                QToolTip.add(defaultPushButton, 
+                QToolTip.add(defaultPushButton,
                     self.__tr('Set to default value of "%1".').arg(default))
 
         elif typ == cups.UI_BANNER_JOB_SHEETS:  # Job sheets widget
             widget = self.getWidget()
-            
+
             layout1 = QGridLayout(widget,1,1,5,10,"layout1")
-    
-            startComboBox = OptionComboBox(0, widget, "startComboBox", group, 
+
+            startComboBox = OptionComboBox(0, widget, "startComboBox", group,
                 "start", choices, default, typ)
-                
+
             layout1.addWidget(startComboBox,0,3)
-    
+
             startTextLabel = QLabel(widget,"startTextLabel")
             layout1.addWidget(startTextLabel,0,2)
-    
+
             endTextLabel = QLabel(widget,"endTextLabel")
             layout1.addWidget(endTextLabel,0,4)
-    
-            endComboBox = OptionComboBox(0, widget, "endComboBox", group, "end", choices, 
+
+            endComboBox = OptionComboBox(0, widget, "endComboBox", group, "end", choices,
                 default, typ, startComboBox)
-                
+
             layout1.addWidget(endComboBox,0,5)
-            
+
             startComboBox.setOther(endComboBox)
-    
-            defaultPushButton = DefaultPushButton(widget, "defaultPushButton", group, option, choices, 
+
+            defaultPushButton = DefaultPushButton(widget, "defaultPushButton", group, option, choices,
                 default, (startComboBox, endComboBox), typ)
-            
+
             layout1.addWidget(defaultPushButton,0,6)
 
             startComboBox.setDefaultPushbutton(defaultPushButton)
             endComboBox.setDefaultPushbutton(defaultPushButton)
-            
+
             textLabel1 = QLabel(widget,"textLabel1")
             layout1.addWidget(textLabel1,0,0)
-            
+
             spacer1 = QSpacerItem(20,20,QSizePolicy.Expanding,QSizePolicy.Minimum)
             layout1.addItem(spacer1,0,1)
-            
+
             textLabel1.setText(text)
             defaultPushButton.setText("Default")
-            
+
             startTextLabel.setText(self.__tr("Start:"))
             endTextLabel.setText(self.__tr("End:"))
 
@@ -919,13 +931,13 @@ class ScrollPrintSettingsView(ScrollView):
                 if value is not None:
                     if d == value[0].lower():
                         s = t
-                    
+
                     if d == value[1].lower():
                         e = t
 
                 if d == default[0].lower():
                     y = t
-                    
+
                 if d == default[1].lower():
                     z = t
 
@@ -934,14 +946,14 @@ class ScrollPrintSettingsView(ScrollView):
 
             if s is not None:
                 startComboBox.setCurrentText(s)
-                
+
             if e is not None:
                 endComboBox.setCurrentText(e)
 
             if value is not None and \
                 value[0].lower() == default[0].lower() and \
                 value[1].lower() == default[1].lower():
-                
+
                 defaultPushButton.setEnabled(False)
 
             if y is not None and z is not None:
@@ -950,40 +962,40 @@ class ScrollPrintSettingsView(ScrollView):
             self.connect(startComboBox, SIGNAL("activated(const QString&)"), self.optionComboBox_activated)
             self.connect(endComboBox, SIGNAL("activated(const QString&)"), self.optionComboBox_activated)
             self.connect(defaultPushButton, SIGNAL("clicked()"), self.defaultPushButton_clicked)
-            
+
         elif typ == cups.PPD_UI_PICKMANY:
             log.error("Unrecognized type: pickmany")
 
         elif typ == cups.UI_UNITS_SPINNER:
             widget = self.getWidget()
-            
+
             layout1 = QHBoxLayout(widget,5,10,"layout1")
 
             textLabel1 = QLabel(widget,"textLabel1")
             layout1.addWidget(textLabel1)
-            
+
             spacer1 = QSpacerItem(20,20,QSizePolicy.Expanding,QSizePolicy.Minimum)
             layout1.addItem(spacer1)
-    
+
             lineEdit1 = QLineEdit(widget,"lineEdit1")
             layout1.addWidget(lineEdit1)
-    
+
             comboBox1 = QComboBox(0,widget,"comboBox1")
             layout1.addWidget(comboBox1)
-    
+
             defaultPushButton = QPushButton(widget,"defaultPushButton")
             layout1.addWidget(defaultPushButton)
-            
+
             textLabel1.setText(text)
             defaultPushButton.setText("Default")
-            
+
         else:
             log.error("Invalid UI value: %s/%s" % (group, option))
 
         if widget is not None:
             self.addWidget(widget, "o:"+option, control)
             return widget
-            
+
 
 
     def __tr(self,s,c = None):
